@@ -2,7 +2,7 @@
                           codegenerator.cpp  -  description
                              -------------------
     begin                : Die Jul 9 2002
-    copyright            : (C) 2002-2015 by Andre Simon
+    copyright            : (C) 2002-2016 by Andre Simon
     email                : andre.simon1@gmx.de
  ***************************************************************************/
 
@@ -130,6 +130,8 @@ CodeGenerator::CodeGenerator ( highlight::OutputType type )
      lineNumberOffset ( 0 ),
      includeStyleDef ( false ),
      lineIndex ( 0 ),
+     syntaxChangeIndex(UINT_MAX),
+     syntaxChangeLineNo(UINT_MAX),
      lineNumberWidth ( 5 ),
      maxLineCnt ( UINT_MAX ),
      terminatingChar ( '\0' ),
@@ -137,7 +139,7 @@ CodeGenerator::CodeGenerator ( highlight::OutputType type )
      formattingEnabled ( false ),
      formattingPossible ( false ),
      validateInput ( false ),
-     numberWrappedLines ( true ),	//until now, wrapped lines were always numbered, so this remains the default.
+     numberWrappedLines ( true ),
      noTrailingNewLine(false),
      resultOfHook(false),
      keywordCase ( StringTools::CASE_UNCHANGED ),
@@ -221,9 +223,9 @@ void CodeGenerator::setStyleOutputPath ( const string& path )
     styleOutputPath = path;
 }
 
-void CodeGenerator::setPluginReadFile ( const string& path )
+void CodeGenerator::setPluginParameter ( const string& param )
 {
-    pluginReadFile = path;
+    pluginParameter = param;
 }
 
 const string&  CodeGenerator::getStyleInputPath()
@@ -247,12 +249,12 @@ bool CodeGenerator::getFragmentCode()
 }
 void CodeGenerator::setKeepInjections ( bool flag )
 {
-  keepInjections=flag;
+    keepInjections=flag;
 }
 
 bool CodeGenerator::getKeepInjections()
 {
-  return keepInjections;
+    return keepInjections;
 }
 void CodeGenerator::setValidateInput ( bool flag )
 {
@@ -379,9 +381,9 @@ void CodeGenerator::reset()
     preFormatter.reset();
     inFile.clear();
     outFile.clear();
-    hostLangDefPath.clear();
     embedLangDefPath.clear();
     printNewLines=true;
+    syntaxChangeIndex = syntaxChangeLineNo = UINT_MAX;
 }
 
 string CodeGenerator::getThemeInitError()
@@ -436,7 +438,7 @@ bool CodeGenerator::readNewLine ( string &newLine )
     return eof || ( lineNumber == maxLineCnt );
 }
 
-void CodeGenerator::matchRegex ( const string &line )
+void CodeGenerator::matchRegex ( const string &line, State skipState, unsigned int offset)
 {
     regexGroups.clear();
     int matchBegin=0;
@@ -446,6 +448,8 @@ void CodeGenerator::matchRegex ( const string &line )
     for ( unsigned int i=0; i<currentSyntax->getRegexElements().size(); i++ ) {
         RegexElement *regexElem = currentSyntax->getRegexElements() [i];
 
+        if (regexElem->open == skipState) continue;
+        
         boost::xpressive::sregex_iterator cur( line.begin(), line.end(), regexElem->rex );
         boost::xpressive::sregex_iterator end;
 
@@ -453,16 +457,15 @@ void CodeGenerator::matchRegex ( const string &line )
             groupID = ( regexElem->capturingGroup<0 ) ? cur->size()-1 : regexElem->capturingGroup;
             matchBegin =  cur->position(groupID);
             regexGroups.insert (
-                make_pair ( matchBegin+1, ReGroup ( regexElem->open, cur->length(groupID), regexElem->kwClass, regexElem->langName ) ) );
+                make_pair ( matchBegin + 1 + offset, ReGroup ( regexElem->open, cur->length(groupID), regexElem->kwClass, regexElem->langName ) ) );
         }
     }
 }
 
 unsigned char CodeGenerator::getInputChar()
 {
-    bool eol = lineIndex == line.length();
-
-    if ( eol ) {
+    // end of line?
+    if ( lineIndex == line.length() ) {
         bool eof=false;
         if ( preFormatter.isEnabled() ) {
             if ( !preFormatter.hasMoreLines() ) {
@@ -492,6 +495,9 @@ unsigned char CodeGenerator::getInputChar()
     return line[lineIndex++];
 }
 
+/** changing this method requires regression testing with nested syntax files (HTML+PHP+JS+CSS, Coffeescript with block regex, Pas + ASM) 
+    especially nested syntax in one line
+ */
 State CodeGenerator::getCurrentState (State oldState)
 {
     unsigned char c='\0';
@@ -514,18 +520,38 @@ State CodeGenerator::getCurrentState (State oldState)
         token= c;
         return _WS;
     }
+        
+    // at this position the syntax change takes place
+    if (lineIndex >= syntaxChangeIndex-1 || syntaxChangeLineNo < lineNumber){
+        loadEmbeddedLang(embedLangDefPath);  // load new syntax                     
+        matchRegex(line);                    // recognize new patterns in the (remaining) line
+        syntaxChangeIndex = syntaxChangeLineNo = UINT_MAX;
+    }
 
+SKIP_EMBEDDED:
+    
     // Test if a regular expression was found at the current position
     if ( !regexGroups.empty() ) {
         if ( regexGroups.count ( lineIndex ) ) {
             token = line.substr ( lineIndex-1, regexGroups[lineIndex].length );
-    
+
             unsigned int oldIndex= lineIndex;
             if ( regexGroups[oldIndex].length>1 ) lineIndex+= regexGroups[oldIndex].length-1;
 
-
-            if ( regexGroups[oldIndex].state==EMBEDDED_CODE_BEGIN) {
-                embedLangDefPath = currentSyntax->getNewPath(regexGroups[oldIndex].name);
+            if ( regexGroups[oldIndex].state==EMBEDDED_CODE_BEGIN /*&& currentSyntax->allowsInnerSection(currentSyntax->getCurrentPath())*/ ) {
+                
+                //do not handle a nested section if the syntax is marked as "sealed" 
+                if (embedLangDefPath.length()==0 || currentSyntax->allowsInnerSection(embedLangDefPath) ) {
+                    embedLangDefPath = currentSyntax->getNewPath(regexGroups[oldIndex].name);
+                    //remember position 
+                    syntaxChangeIndex = lineIndex+2;
+                    syntaxChangeLineNo = lineNumber;
+                }
+                
+                // repeat parsing of this line without nested state recognition to highlight opening delimiter in the host syntax
+                matchRegex(line, EMBEDDED_CODE_BEGIN);
+                lineIndex = oldIndex;
+                goto SKIP_EMBEDDED; // this is how it should be done
             }
 
             if ( regexGroups[oldIndex].state==IDENTIFIER_BEGIN || regexGroups[oldIndex].state==KEYWORD ) {
@@ -545,7 +571,6 @@ State CodeGenerator::getCurrentState (State oldState)
     return STANDARD;
 }
 
-
 State CodeGenerator::validateState(State newState, State oldState, unsigned int kwClass)
 {
 
@@ -563,10 +588,10 @@ State CodeGenerator::validateState(State newState, State oldState, unsigned int
         resultOfHook = res.size()>=1;
         if (resultOfHook) {
             State validatedState = (State)res[0].asNumber();
-            if ( validatedState== _REJECT){
+            if ( validatedState== _REJECT) {
                 // proceed using only the first character of the token
                 lineIndex -= (token.length() -1);
-                token=token.substr(0, 1); 
+                token=token.substr(0, 1);
                 return oldState;
             }
             return validatedState;
@@ -585,14 +610,15 @@ void CodeGenerator::maskString ( ostream& ss, const string & s )
 }
 
 
-Diluculum::LuaValueList CodeGenerator::callDecorateFct(const string&token){
+Diluculum::LuaValueList CodeGenerator::callDecorateFct(const string&token)
+{
     Diluculum::LuaValueList params;
     params.push_back(Diluculum::LuaValue(token));
     params.push_back(Diluculum::LuaValue(currentState));
     params.push_back(Diluculum::LuaValue(currentKeywordClass));
 
     return currentSyntax->getLuaState()->call ( *currentSyntax->getDecorateFct(),
-                                               params,"getDecorateFct call")  ;
+            params,"getDecorateFct call")  ;
 }
 
 void CodeGenerator::printMaskedToken (bool flushWhiteSpace, StringTools::KeywordCase tcase )
@@ -601,15 +627,15 @@ void CodeGenerator::printMaskedToken (bool flushWhiteSpace, StringTools::Keyword
         flushWs();
     string caseToken = StringTools::change_case ( token, tcase );
     if (currentSyntax->getDecorateFct()) {
-        
-      Diluculum::LuaValueList res=callDecorateFct(caseToken);    
+
+        Diluculum::LuaValueList res=callDecorateFct(caseToken);
         if (res.size()==1) {
             *out<<res[0].asString();
         } else {
             maskString ( *out, caseToken );
         }
     } else {
-      maskString ( *out, caseToken );
+        maskString ( *out, caseToken );
     }
 
     token.clear();
@@ -667,15 +693,23 @@ bool CodeGenerator::initIndentationScheme ( const string &indentScheme )
         formatter->setFormattingStyle ( astyle::STYLE_LISP );
     } else if ( indentScheme=="vtk") {
         formatter->setFormattingStyle ( astyle::STYLE_VTK );
+    } else if ( indentScheme=="mozilla") {
+        formatter->setFormattingStyle ( astyle::STYLE_MOZILLA );
     } else {
         return false;
     }
     return formattingEnabled=true;
 }
 
-LoadResult CodeGenerator::loadLanguage ( const string& langDefPath )
+LoadResult CodeGenerator::loadLanguage ( const string& langDefPath, bool embedded )
 {
 
+    if (!embedded) {
+        while (!nestedLangs.empty()) {
+            nestedLangs.pop();
+        }   
+    }
+    
     bool reloadNecessary= currentSyntax ? currentSyntax->needsReload ( langDefPath ): true;
     LoadResult result=LOAD_OK;
 
@@ -685,7 +719,7 @@ LoadResult CodeGenerator::loadLanguage ( const string& langDefPath )
             result=LOAD_OK;
         } else {
             currentSyntax=new SyntaxReader();
-            result=currentSyntax->load(langDefPath, pluginReadFile, outputType);
+            result=currentSyntax->load(langDefPath, pluginParameter, outputType);
             syntaxReaders[langDefPath]=currentSyntax;
         }
 
@@ -765,20 +799,22 @@ bool CodeGenerator::validateInputStream()
            || magic_table[magic_index] == magic_utf8;
 }
 
-void CodeGenerator::printHeader(){
-  if ( ! fragmentOutput ) 
-    *out << getHeader();
-  
-  if ( !fragmentOutput || keepInjections) 
-    *out << currentSyntax->getHeaderInjection();
+void CodeGenerator::printHeader()
+{
+    if ( ! fragmentOutput )
+        *out << getHeader();
+
+    if ( !fragmentOutput || keepInjections)
+        *out << currentSyntax->getHeaderInjection();
 }
 
-void CodeGenerator::printFooter(){
-  if ( !fragmentOutput || keepInjections) 
-    *out << currentSyntax->getFooterInjection();
-  
-  if ( ! fragmentOutput )
-    *out << getFooter();
+void CodeGenerator::printFooter()
+{
+    if ( !fragmentOutput || keepInjections)
+        *out << currentSyntax->getFooterInjection();
+
+    if ( ! fragmentOutput )
+        *out << getFooter();
 }
 
 ParseError CodeGenerator::generateFile ( const string &inFileName,
@@ -890,7 +926,7 @@ string CodeGenerator::generateStringFromFile ( const string &inFileName )
     printHeader();
     printBody();
     printFooter();
-    
+
     string result = static_cast<ostringstream*> ( out )->str();
 
     delete out;
@@ -937,12 +973,13 @@ void CodeGenerator::closeKWTag ( unsigned int kwClassID )
 
 bool CodeGenerator::loadEmbeddedLang(const string&embedLangDefPath)
 {
-    //save path of host language
-    if (hostLangDefPath.empty()) {
-        hostLangDefPath =currentSyntax->getCurrentPath();
+    if (nestedLangs.empty()) {
+        nestedLangs.push(currentSyntax->getCurrentPath() );
     }
-    //load syntax of embedded langage
-    LoadResult res = loadLanguage(embedLangDefPath);
+    if (nestedLangs.top() != embedLangDefPath) {
+        nestedLangs.push(embedLangDefPath);
+    }
+    LoadResult res = loadLanguage(embedLangDefPath, true);
     //pass end delimiter regex to syntax description
     currentSyntax->restoreLangEndDelim(embedLangDefPath);
     return res == LOAD_OK;
@@ -1021,20 +1058,20 @@ void CodeGenerator::processRootState()
             eof=processSymbolState();
             openTag ( STANDARD );
             break;
-        case EMBEDDED_CODE_BEGIN:
+
         case EMBEDDED_CODE_END:
             closeTag ( STANDARD );
             eof=processSyntaxChangeState(state);
             openTag ( STANDARD );
             break;
         case _EOL:
-            
+
             // XTERM256 fix (issue with less cmd)
             if  (!firstLine || showLineNumbers) {
-                closeTag ( STANDARD ); 
+                closeTag ( STANDARD );
             }
             insertLineNumber ( !firstLine );
-            if (!firstLine || showLineNumbers) { 
+            if (!firstLine || showLineNumbers) {
                 flushWs();
                 openTag ( STANDARD );
             }
@@ -1052,14 +1089,14 @@ void CodeGenerator::processRootState()
         }
     } while ( !eof );
     closeTag ( STANDARD );
-    
+
     if (currentSyntax->getDecorateLineEndFct()) {
-      Diluculum::LuaValueList res=callDecorateLineFct(false);    
-      if (res.size()==1) {
-        *out << res[0].asString();
-      } 
-    } 
-    
+        Diluculum::LuaValueList res=callDecorateLineFct(false);
+        if (res.size()==1) {
+            *out << res[0].asString();
+        }
+    }
+
     printNewLines = !noTrailingNewLine;
     *out << getNewLine();
     *out << flush;
@@ -1070,24 +1107,24 @@ bool CodeGenerator::processSyntaxChangeState(State myState)
     State newState=STANDARD;
     bool eof=false,
          exitState=false;
+
     openTag ( KEYWORD );
     do {
-        if (myState==EMBEDDED_CODE_BEGIN) {
-            if (!loadEmbeddedLang(embedLangDefPath)) {
-                // exit or segfault
-                return true;
-            }
-            //test current line again to match tokens of the embedded language
-            matchRegex(line);
-        } else if (myState==EMBEDDED_CODE_END) {
-            // load host language syntax
-            loadLanguage(hostLangDefPath);
-            //test current line again to match tokens of the host language
-            matchRegex(line);
-        }
 
+        if (myState==EMBEDDED_CODE_END) {
+            if (!nestedLangs.empty()) {
+                nestedLangs.pop();
+            }
+            // load host language syntax
+            if (!nestedLangs.empty()) {
+                loadLanguage(nestedLangs.top(), true);
+            }
+        }
+        
         printMaskedToken ( newState!=_WS );
+
         newState= getCurrentState(myState);
+
         switch ( newState ) {
         case _WS:
             processWsState();
@@ -1104,11 +1141,12 @@ bool CodeGenerator::processSyntaxChangeState(State myState)
             break;
         }
     } while (  !exitState  &&  !eof );
-
     closeTag ( KEYWORD );
+
     return eof;
 }
 
+
 bool CodeGenerator::processKeywordState ( State myState )
 {
     State newState=STANDARD;
@@ -1291,10 +1329,13 @@ bool CodeGenerator::processDirectiveState()
             break;
         case _EOL:
             printMaskedToken();
+            
             if ( preFormatter.isEnabled() && preFormatter.isWrappedLine ( lineNumber-1 ) ) {
                 exitState=false;
             } else {
-                exitState= ( terminatingChar!=currentSyntax->getContinuationChar() );
+                if (currentSyntax->getContinuationChar()!=0x13){
+                    exitState= ( terminatingChar!=currentSyntax->getContinuationChar() );
+                } 
             }
             if ( !exitState ) wsBuffer += closeTags[DIRECTIVE];
             insertLineNumber();
@@ -1370,11 +1411,11 @@ bool CodeGenerator::processStringState ( State oldState )
             break;
         case STRING_END:
             if (resultOfHook || currentSyntax->matchesOpenDelimiter (token,  STRING_END, openDelimID)) {
-              if (currentSyntax->assertDelimEqualLength()){
-                exitState= openDelim.length()==token.length();
-              }else {
-                exitState= true;
-              }
+                if (currentSyntax->assertDelimEqualLength()) {
+                    exitState= openDelim.length()==token.length();
+                } else {
+                    exitState= true;
+                }
                 printMaskedToken();
             }
             break;
@@ -1512,17 +1553,18 @@ void CodeGenerator::processWsState()
         token.clear();
         return;
     }
+
     flushWs();
+
     int cntWs=0;
     lineIndex--;
 
-    // while (iswspace(line[lineIndex])  ) {
     while ( line[lineIndex]==' ' || line[lineIndex]=='\t' ) {
         ++cntWs;
         ++lineIndex;
     }
-
     if ( cntWs>1 ) {
+
         unsigned int styleID=getStyleID ( currentState, currentKeywordClass );
         if ( excludeWs && styleID!=_UNKNOWN ) {
             *out << closeTags[styleID];
@@ -1552,38 +1594,39 @@ string CodeGenerator::getNewLine()
     return (printNewLines) ? newLineTag : "";
 }
 
-Diluculum::LuaValueList CodeGenerator::callDecorateLineFct(bool isLineStart){
-       
-   Diluculum::LuaValueList params;
-   params.push_back(Diluculum::LuaValue(lineNumber));
+Diluculum::LuaValueList CodeGenerator::callDecorateLineFct(bool isLineStart)
+{
+
+    Diluculum::LuaValueList params;
+    params.push_back(Diluculum::LuaValue(lineNumber));
+
+    return currentSyntax->getLuaState()->call ( isLineStart ?
+            *currentSyntax->getDecorateLineBeginFct(): *currentSyntax->getDecorateLineEndFct(),
+            params,"getDecorateLineFct call")  ;
 
-    return currentSyntax->getLuaState()->call ( isLineStart ? 
-           *currentSyntax->getDecorateLineBeginFct(): *currentSyntax->getDecorateLineEndFct(),
-           params,"getDecorateLineFct call")  ;
-  
 }
 
 void CodeGenerator::insertLineNumber ( bool insertNewLine )
 {
     if ( insertNewLine ) {
-      if (currentSyntax->getDecorateLineEndFct()) {
-        Diluculum::LuaValueList res=callDecorateLineFct(false);    
-        if (res.size()==1) {
-          wsBuffer +=res[0].asString();
-        } 
-      } 
-      
-      wsBuffer += getNewLine();
+        if (currentSyntax->getDecorateLineEndFct()) {
+            Diluculum::LuaValueList res=callDecorateLineFct(false);
+            if (res.size()==1) {
+                wsBuffer +=res[0].asString();
+            }
+        }
+
+        wsBuffer += getNewLine();
     }
 
-    
+
     if (currentSyntax->getDecorateLineBeginFct()) {
-      Diluculum::LuaValueList res=callDecorateLineFct(true);    
-      if (res.size()==1) {
-        wsBuffer +=res[0].asString();
-      } 
-    } 
-    
+        Diluculum::LuaValueList res=callDecorateLineFct(true);
+        if (res.size()==1) {
+            wsBuffer +=res[0].asString();
+        }
+    }
+
     if ( showLineNumbers ) {
         ostringstream os;
         ostringstream numberPrefix;
@@ -1621,11 +1664,7 @@ bool CodeGenerator::printExternalStyle ( const string &outFile )
                         << HIGHLIGHT_VERSION << ", " << HIGHLIGHT_URL
                         << " " << styleCommentClose << "\n";
 
-            *cssOutFile << "\n" << styleCommentOpen
-                        << " Highlighting theme: "
-                        << docStyle.getDescription() <<" "
-                        << styleCommentClose << "\n\n"
-                        << getStyleDefinition()
+            *cssOutFile << getStyleDefinition()
                         << "\n";
             *cssOutFile << readUserStyleDef();
             if ( !outFile.empty() ) delete cssOutFile;
@@ -1672,12 +1711,12 @@ bool CodeGenerator::initPluginScript(const string& script)
     if (script.empty()) return true;
 
     try {
-         
+
         userScriptError="";
         Diluculum::LuaState ls;
         ls.doFile (script);
         int listIdx=1;
-        
+
         while (ls["Plugins"][listIdx].value() !=Diluculum::Nil) {
 
             // Theme plugins
