@@ -2,7 +2,7 @@
                           syntaxreader.cpp  -  description
                              -------------------
     begin                : Wed Nov 28 2001
-    copyright            : (C) 2001-2015 by Andre Simon
+    copyright            : (C) 2001-2016 by Andre Simon
     email                : andre.simon1@gmx.de
  ***************************************************************************/
 
@@ -32,6 +32,7 @@ along with Highlight.  If not, see <http://www.gnu.org/licenses/>.
 namespace highlight
 {
 
+//default expressions, can be overridden by syntax definition
 const string SyntaxReader::REGEX_IDENTIFIER =
     "[a-zA-Z_]\\w*";
 
@@ -41,7 +42,10 @@ const string SyntaxReader::REGEX_NUMBER =
 const string SyntaxReader::REGEX_ESCSEQ =
     "\\\\u[[:xdigit:]]{4}|\\\\\\d{3}|\\\\x[[:xdigit:]]{2}|\\\\[ntvbrfa\\\\\\?'\"]";
 
-EmbedLangDelimMap SyntaxReader::exitDelimiters;
+DelimiterMap SyntaxReader::nestedStateEndDelimiters;
+
+AllowInnerSectionsMap SyntaxReader::allowInnerSections;
+
 vector<Diluculum::LuaFunction*> SyntaxReader::pluginChunks;
 
 int RegexElement::instanceCnt=0;
@@ -158,7 +162,7 @@ LoadResult SyntaxReader::load ( const string& langDefPath, const string& pluginR
         SyntaxReader **s = (SyntaxReader **)lua_newuserdata(ls.getState(), sizeof(SyntaxReader *));
         *s=this;
         lua_setglobal(ls.getState(), GLOBAL_SR_INSTANCE_NAME);
-    
+
         // ececute script and read values
         ls.doFile (langDefPath);
 
@@ -225,10 +229,10 @@ LoadResult SyntaxReader::load ( const string& langDefPath, const string& pluginR
 
                     delimiterDistinct[openDelimId]=openDelim!=closeDelim;
                     delimiterDistinct[closeDelimId]=openDelim!=closeDelim;
-                    delimIds2[closeDelimId]=openDelimId;
+                    matchingDelimiters[closeDelimId]=openDelimId;
 
                 } else {
-                    regex.push_back ( new RegexElement ( SL_COMMENT, SL_COMMENT_END, 
+                    regex.push_back ( new RegexElement ( SL_COMMENT, SL_COMMENT_END,
                                                          StringTools::trim(ls["Comments"][listIdx]["Delimiter"][1].value().asString()), 0, -1 ) );
                 }
                 ++listIdx;
@@ -258,7 +262,7 @@ LoadResult SyntaxReader::load ( const string& langDefPath, const string& pluginR
                 regex.push_back (elem );
             }
             if (ls["Strings"]["Interpolation"].value()!=Diluculum::Nil) {
-                RegexElement* elem=new RegexElement ( STRING_INTERPOLATION, STRING_INTERPOLATION_END, 
+                RegexElement* elem=new RegexElement ( STRING_INTERPOLATION, STRING_INTERPOLATION_END,
                                                       StringTools::trim( ls["Strings"]["Interpolation"].value().asString()), 0, -1 );
                 regex.push_back (elem );
             }
@@ -282,7 +286,7 @@ LoadResult SyntaxReader::load ( const string& langDefPath, const string& pluginR
                     closeDelimId=elem->instanceId;
                     regex.push_back( elem );
 
-                    delimIds2[closeDelimId]=openDelimId;
+                    matchingDelimiters[closeDelimId]=openDelimId;
 
                     if (ls["Strings"]["DelimiterPairs"][listIdx]["Raw"].value()!=Diluculum::Nil) {
                         rawStringOpenDelims[openDelimId]=ls["Strings"]["DelimiterPairs"][listIdx]["Raw"].value().asBoolean();
@@ -290,7 +294,7 @@ LoadResult SyntaxReader::load ( const string& langDefPath, const string& pluginR
                     ++listIdx;
                 }
             }
-            
+
             assertEqualLength=readFlag(ls["Strings"]["AssertEqualLength"]);
 
             string escRegex=(ls["Strings"]["Escape"].value()==Diluculum::Nil)?REGEX_ESCSEQ:ls["Strings"]["Escape"].value().asString();
@@ -318,12 +322,20 @@ LoadResult SyntaxReader::load ( const string& langDefPath, const string& pluginR
                 string lang= ls["NestedSections"][listIdx]["Lang"].value().asString();
                 string openDelim=StringTools::trim(ls["NestedSections"][listIdx]["Delimiter"][1].value().asString());
                 regex.insert(regex.begin(), 1, new RegexElement(EMBEDDED_CODE_BEGIN, EMBEDDED_CODE_BEGIN, openDelim, 0, -1, lang));
-
+                
                 string closeDelim=StringTools::trim(ls["NestedSections"][listIdx]["Delimiter"][2].value().asString());
-                exitDelimiters[getNewPath(lang)] = closeDelim;
-
+                nestedStateEndDelimiters[getNewPath(lang)] = closeDelim;
+                
+                bool allowInnerSectionsFlag=true;
+                if (ls["NestedSections"][listIdx]["Sealed"].value()!=Diluculum::Nil){
+                    allowInnerSectionsFlag = !ls["NestedSections"][listIdx]["Sealed"].value().asBoolean();
+                }
+                allowInnerSections[getNewPath(lang)] = allowInnerSectionsFlag;
+                
                 ++listIdx;
             }
+            //allow host syntax
+            allowInnerSections[getCurrentPath()] = true;               
         }
 
         if (globals.count("HeaderInjection")) {
@@ -347,7 +359,7 @@ LoadResult SyntaxReader::load ( const string& langDefPath, const string& pluginR
         if (globals.count("DecorateLineEnd")) {
             decorateLineEndFct=new Diluculum::LuaFunction(ls["DecorateLineEnd"].value().asFunction());
         }
-        
+
     } catch (Diluculum::LuaError err) {
         luaErrorMsg = string(err.what());
         return LOAD_FAILED_LUA;
@@ -387,27 +399,30 @@ int SyntaxReader::luaAddKeyword (lua_State *L)
 
 void SyntaxReader::restoreLangEndDelim(const string& langPath)
 {
-    //TODO exitDelimiters be left static?
-    if ( !langPath.empty()&& exitDelimiters.count(langPath) ) {
-        regex.insert (regex.begin(),1, new RegexElement ( EMBEDDED_CODE_END,EMBEDDED_CODE_END, exitDelimiters[langPath] ) );
+    if ( !langPath.empty()&& nestedStateEndDelimiters.count(langPath) ) {
+        regex.insert (regex.begin(),1, new RegexElement ( EMBEDDED_CODE_END,EMBEDDED_CODE_END, nestedStateEndDelimiters[langPath] ) );
     }
 }
 
+bool SyntaxReader::allowsInnerSection(const string& langPath)
+{
+    return allowInnerSections[langPath];
+}
+
 unsigned int SyntaxReader::generateNewKWClass ( int classID )
 {
     char className[5]= {0};
-    snprintf(className, sizeof(className), "kw%c", ('a'+classID-1)); 
-    
+    snprintf(className, sizeof(className), "kw%c", ('a'+classID-1));
+
     unsigned int newClassID=0;
     bool found=false;
     while (!keywordClasses.empty() && newClassID<keywordClasses.size() && !found ) {
-      found = ( className==keywordClasses.at(newClassID++) );
+        found = ( className==keywordClasses.at(newClassID++) );
     }
     if ( !found ) {
         newClassID++;
         keywordClasses.push_back ( className );
     }
-
     return newClassID;
 }
 
@@ -448,7 +463,7 @@ bool SyntaxReader::matchesOpenDelimiter ( const string& token, State s, int open
         RegexElement *regexElem = getRegexElements() [i];
         if (regexElem->open==s ) {
 
-            if( regex_match( token, what, regexElem->rex ) && delimIds2[regexElem->instanceId]==openDelimId) {
+            if( regex_match( token, what, regexElem->rex ) && matchingDelimiters[regexElem->instanceId]==openDelimId) {
                 return true;
             }
 
