@@ -31,6 +31,7 @@ along with Highlight.  If not, see <http://www.gnu.org/licenses/>.
 #include <sstream>
 #include <string>
 #include <iomanip>
+#include <stack>
 
 #include "syntaxreader.h"
 #include "themereader.h"
@@ -155,15 +156,13 @@ public:
         \param indentScheme Name of indentation scheme
         \return true if successfull
      */
-    bool initIndentationScheme ( const string&indentScheme );
+    bool initIndentationScheme ( const string& indentScheme );
 
     /** \param langDefPath Absolute path to language definition, may be used multiple times for a generator instance
-        \return   LOAD_OK,
-    	      LOAD_FAILED,
-    	      LOAD_FAILED_REGEX,
-    	      LOAD_FAILED_LUA
+     *  \param embedded set True if method is called to laod an embedded language
+        \return   LOAD_OK, LOAD_FAILED, LOAD_FAILED_REGEX, LOAD_FAILED_LUA
     */
-    LoadResult loadLanguage ( const string& langDefPath );
+    LoadResult loadLanguage ( const string& langDefPath, bool embedded=false );
 
     /**
      Generate output file from input file
@@ -347,9 +346,9 @@ public:
     */
     void setStartingNestedLang(const string &langName);
 
-    /** \param path path of plugin input file
+    /** \param param path of plugin input file
     */
-    void setPluginReadFile ( const string& path );
+    void setPluginParameter ( const string& param );
 
     /** Load the Lua functions og the plug-in script
     	\param script path of the plug-in script
@@ -543,6 +542,9 @@ protected:
     /** Current state*/
     State currentState;
 
+    /** State to utput nested code delimiters */
+   // State nestedCodeDelimState;
+    
     /** keyword class id, used to apply the corresponding keyword style*/
     unsigned int currentKeywordClass;
 
@@ -580,12 +582,6 @@ protected:
         return StringTools::change_case ( encoding ) !="none";
     }
 
-    /** add Lua state variables specific to the output format */
-    /*
-    virtual void addSpecificVars()
-    {
-    };*/
-
     /** Invoke plugin decorate user function */
     Diluculum::LuaValueList callDecorateFct(const string& token);
     
@@ -672,7 +668,7 @@ private:
     string embedLangDefPath;
 
     /// path to plugin input file
-    string pluginReadFile;
+    string pluginParameter;
 
     /// name of nested language which starts the input (ie opening delim missing, but closing delim exists)
     string embedLangStart;
@@ -680,6 +676,12 @@ private:
     /// contains current position in line
     unsigned int lineIndex;
 
+    /// line index where syntax change takes place
+    unsigned int syntaxChangeIndex;
+
+    /// line number where syntax change takes place
+    unsigned int syntaxChangeLineNo;
+
     /// width of line numbering coloumn
     unsigned int lineNumberWidth;
 
@@ -738,6 +740,8 @@ private:
 
     /** return new state */
     State getCurrentState (State oldState);
+    
+    std::stack<std::string> nestedLangs;
 
     /* Methods that represent a parsing state */
     bool processKeywordState ( State myState );  ///< process keywords
@@ -768,8 +772,9 @@ private:
     map <string, SyntaxReader*> syntaxReaders;
 
     /** test for regular expressions
-        \param line current input line*/
-    void matchRegex ( const string &line );
+        \param line current input line
+        \param skipState state which should be ignored*/
+    void matchRegex ( const string &line, State skipState=_UNKNOWN, unsigned int offset=0 );
 
     /** \return true if input is no binary stream */
     bool validateInputStream();
